const helpLink = "<br>ONYX/scry v0.2.0 [ <a target='_blank' href='https://nilfm.cc/git/onyx-scry/about/LICENSE'>license</a> | <a target='_blank' href='https://nilfm.cc/git/onyx-scry/about'>manual</a> ]";


function init(): void {
  let overlays: OverlayState = new OverlayState();
  try {
    overlays = OverlayState.load();
  } catch {
    alert("Error reading saved data; initializing with empty data.");
  }
  const map = L.map('map').fitWorld();
  
  const streetLayer = TileLayerWrapper.constructLayer(
    "streetLayer",
    L.tileLayer(
      'https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
      {
        maxZoom: 19,
        attribution: "street map data &copy; OpenStreetMap contributors" + helpLink
      }));
      
  const satelliteLayer = TileLayerWrapper.constructLayer(
    "satelliteLayer",
    L.tileLayer(
      'https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}',
      {
        maxZoom: 19,
        attribution: "satellite data &copy; Esri" + helpLink
      }));
      
  TileLayerWrapper.enableOnly("streetLayer", map);
  
  overlays.markers.forEach(m=>m.add(map));
  overlays.circles.forEach(m=>m.add(map));
  overlays.paths.forEach(m=>m.add(map));
  overlays.polyline.add(map);

  const modals = new ModalCollection(
    new CreateOverlayModal(),
    new CancelModal(),
    new OKCancelModal(),
    new InfoModal(),
    new OverlayManagementModal(),
    new ImportExportModal());

  MapHandler.init(map, overlays, TileLayerWrapper.layers, modals);
  
  MapHandler.setButtonClick("home-btn", MapHandler.goHome);
  MapHandler.setButtonClick("addPoint-btn", MapHandler.markerCollect);
  MapHandler.setButtonClick("addCircle-btn", MapHandler.circleCollect);
  MapHandler.setButtonClick("addPolygon-btn", MapHandler.polygonCollect);
  
  MapHandler.setButtonClick("tiles-btn", MapHandler.swapTiles);  
  MapHandler.setButtonClick("save-btn", MapHandler.overlaySave);
  MapHandler.setButtonClick("clear-btn", MapHandler.overlayClear);
  MapHandler.setButtonClick("restore-btn", MapHandler.overlayReset);
  MapHandler.setButtonClick("menu-btn", MapHandler.toggleMenu);
  
  MapHandler.setButtonClick("set-home-btn", MapHandler.setHome);
  MapHandler.setButtonClick("export-all-btn", MapHandler.exportAll);
  MapHandler.setButtonClick("import-export-cancel-btn", MapHandler.closeImportExport);
  MapHandler.setButtonClick("import-btn", MapHandler.import);
  
  map.on("locationfound", MapHandler.setHome);
  
  map.on("locationerror", ()=> {
    const info = modals.info;
    info.setMsg("Could not get location data");
    info.setVisible(true);
  });
  
  // the menu doesn't open on the first click unless we do this first... not sure why
  modals.closeAll();
  
  const homeData = localStorage.getItem("home");
  if (homeData) {
    const home = <Point>JSON.parse(homeData);
    map.setView(home, 13);
  } else {
    const okCancel = modals.okCancel;
    const okBtn = okCancel.okBtn();
    if (okBtn) {
      okBtn.onclick = () => {
        modals.closeAll();
        map.locate({setView: true, maxZoom: 13});     
      }
    }
    const cancelBtn = okCancel.cancelBtn();
    if (cancelBtn) {
      cancelBtn.onclick = () => {
        modals.closeAll();
      }
    }
    okCancel.setMsg("Would you like to use location data to set Home?");
    okCancel.setVisible(true);
  }
}

init();
